@@ -22,13 +22,15 @@ from oslo_config import cfg
 from oslo_log import log
 from oslo_utils import timeutils
 import requests
+from six import PY3
 
-from monasca_common.rest import utils as rest_utils
+from monasca_api.common.rest import utils as rest_utils
 
 from monasca_api.common.repositories import exceptions
 from monasca_api.common.repositories import metrics_repository
 
 MEASUREMENT_NOT_FOUND_MSG = "measurement not found"
+DATABASE_NOT_FOUND_MSG = "database not found"
 
 CONF = cfg.CONF
 LOG = log.getLogger(__name__)
@@ -41,8 +43,7 @@ class MetricsRepository(metrics_repository.AbstractMetricsRepository):
             self.conf = cfg.CONF
             self.influxdb_client = client.InfluxDBClient(
                 self.conf.influxdb.ip_address, self.conf.influxdb.port,
-                self.conf.influxdb.user, self.conf.influxdb.password,
-                self.conf.influxdb.database_name)
+                self.conf.influxdb.user, self.conf.influxdb.password)
             self._version = None
             self._init_version()
         except Exception as ex:
@@ -50,8 +51,6 @@ class MetricsRepository(metrics_repository.AbstractMetricsRepository):
             raise exceptions.RepositoryException(ex)
 
         self._serie_builders_version_map = {
-            'to_0.11.0': (self._build_serie_dimension_values_to_v0_11_0,
-                          self._build_serie_metric_list_to_v0_11_0),
             'from_0.11.0': (self._build_serie_dimension_values_from_v0_11_0,
                             self._build_serie_metric_list_from_v0_11_0)}
 
@@ -89,8 +88,7 @@ class MetricsRepository(metrics_repository.AbstractMetricsRepository):
         try:
             influxdb_version = self._get_influxdb_version()
             if influxdb_version < version.StrictVersion('0.11.0'):
-                self._version = 'to_0.11.0'
-                LOG.info('Initialize InfluxDB serie builders <  v0.11.0')
+                raise Exception('Influxdb < v0.11.0 is not supported')
             else:
                 self._version = 'from_0.11.0'
                 LOG.info('Initialize InfluxDB serie builders >=  v0.11.0')
@@ -101,18 +99,27 @@ class MetricsRepository(metrics_repository.AbstractMetricsRepository):
             LOG.warning('influxdb is not ready yet')
         except Exception as ex:
             LOG.exception(ex)
-            # Initialize the serie builders to v0_11_0. Not sure when SHOW DIAGNOSTICS added
-            # support for a version string so to address backward compatibility initialize
-            # InfluxDB serie builders <  v0.11.0
-            self._version = 'to_0.11.0'
-            LOG.info('Initialize InfluxDB serie builders <  v0.11.0')
+            self._version = 'from_0.11.0'
+            LOG.info('Initialize InfluxDB serie builders >=  v0.11.0')
 
     def _get_influxdb_version(self):
-        '''If Version found in the result set, return the InfluxDB Version,
+        '''New versions of InfluxDB implements the method ping() which returns
+        the version. In case that the method isn't present, the query SHOW DIAGNOSTICS
+        will be used.
+        If Version found in the result set, return the InfluxDB Version,
         otherwise raise an exception. InfluxDB has changed the format of their
         result set and SHOW DIAGNOSTICS was introduced at some point so earlier releases
         of InfluxDB might not return a Version.
         '''
+        try:
+            result = self.influxdb_client.ping()
+            LOG.info("Found Influxdb version {0}".format(result))
+            return version.StrictVersion(result)
+        except Exception as ex:
+            LOG.warn(ex)
+            LOG.warn("Getting version from method ping failed,"
+                     " now trying with SHOW DIAGNOSTICS")
+
         try:
             result = self.influxdb_client.query('SHOW DIAGNOSTICS')
         except InfluxDBClientError as ex:
@@ -158,7 +165,8 @@ class MetricsRepository(metrics_repository.AbstractMetricsRepository):
         return query
 
     def _build_show_tag_values_query(self, metric_name, dimension_name,
-                                     tenant_id, region):
+                                     tenant_id, region, start_timestamp,
+                                     end_timestamp):
         from_with_clause = ''
         if metric_name:
             from_with_clause += ' from "{}"'.format(metric_name)
@@ -166,18 +174,21 @@ class MetricsRepository(metrics_repository.AbstractMetricsRepository):
         if dimension_name:
             from_with_clause += ' with key = "{}"'.format(dimension_name)
 
-        where_clause = self._build_where_clause(None, None, tenant_id, region)
+        where_clause = self._build_where_clause(None, None, tenant_id, region,
+                                                start_timestamp, end_timestamp)
 
         query = 'show tag values' + from_with_clause + where_clause
 
         return query
 
-    def _build_show_tag_keys_query(self, metric_name, tenant_id, region):
+    def _build_show_tag_keys_query(self, metric_name, tenant_id, region,
+                                   start_timestamp, end_timestamp):
         from_with_clause = ''
         if metric_name:
             from_with_clause += ' from "{}"'.format(metric_name)
 
-        where_clause = self._build_where_clause(None, None, tenant_id, region)
+        where_clause = self._build_where_clause(None, None, tenant_id, region,
+                                                start_timestamp, end_timestamp)
 
         query = 'show tag keys' + from_with_clause + where_clause
 
@@ -250,39 +261,43 @@ class MetricsRepository(metrics_repository.AbstractMetricsRepository):
         # name - optional
         if name:
             # replace ' with \' to make query parsable
-            clean_name = name.replace("'", "\\'")
-            where_clause += ' from  "{}" '.format(clean_name.encode('utf8'))
-
-        # tenant id
-        where_clause += " where _tenant_id = '{}' ".format(tenant_id.encode(
-            "utf8"))
+            clean_name = name.replace("'", "\\'") if PY3 \
+                else name.replace("'", "\\'").encode('utf-8')
+            where_clause += ' from  "{}" '.format(clean_name)
 
         # region
-        where_clause += " and _region = '{}' ".format(region.encode('utf8'))
+        where_clause += " where _region = '{}'".format(region)
+
+        # tenant id
+        if not self.conf.influxdb.db_per_tenant:
+            where_clause += " and _tenant_id = '{}'".format(tenant_id)
 
         # dimensions - optional
         if dimensions:
             for dimension_name, dimension_value in iter(
                     sorted(dimensions.items())):
                 # replace ' with \' to make query parsable
-                clean_dimension_name = dimension_name.replace("\'", "\\'")
+                clean_dimension_name = dimension_name.replace("\'", "\\'") if PY3 \
+                    else dimension_name.replace("\'", "\\'").encode('utf-8')
                 if dimension_value == "":
-                    where_clause += " and \"{}\" =~ /.+/ ".format(
+                    where_clause += " and \"{}\" =~ /.+/".format(
                         clean_dimension_name)
                 elif '|' in dimension_value:
                     # replace ' with \' to make query parsable
-                    clean_dimension_value = dimension_value.replace("\'", "\\'")
+                    clean_dimension_value = dimension_value.replace("\'", "\\'") if PY3 else \
+                        dimension_value.replace("\'", "\\'").encode('utf-8')
 
-                    where_clause += " and \"{}\" =~ /^{}$/ ".format(
-                        clean_dimension_name.encode('utf8'),
-                        clean_dimension_value.encode('utf8'))
+                    where_clause += " and \"{}\" =~ /^{}$/".format(
+                        clean_dimension_name,
+                        clean_dimension_value)
                 else:
                     # replace ' with \' to make query parsable
-                    clean_dimension_value = dimension_value.replace("\'", "\\'")
+                    clean_dimension_value = dimension_value.replace("\'", "\\'") if PY3 else \
+                        dimension_value.replace("\'", "\\'").encode('utf-8')
 
-                    where_clause += " and \"{}\" = '{}' ".format(
-                        clean_dimension_name.encode('utf8'),
-                        clean_dimension_value.encode('utf8'))
+                    where_clause += " and \"{}\" = '{}'".format(
+                        clean_dimension_name,
+                        clean_dimension_value)
 
         if start_timestamp is not None:
             where_clause += " and time >= " + str(int(start_timestamp *
@@ -302,6 +317,19 @@ class MetricsRepository(metrics_repository.AbstractMetricsRepository):
                                                end_timestamp)
         return from_clause
 
+    def query_tenant_db(self, query, tenant_id):
+        database = ('%s_%s' % (self.conf.influxdb.database_name, tenant_id)
+                    if self.conf.influxdb.db_per_tenant
+                    else self.conf.influxdb.database_name)
+        try:
+            return self.influxdb_client.query(query, database=database)
+        except InfluxDBClientError as ex:
+            if (str(ex).startswith(DATABASE_NOT_FOUND_MSG) and
+                    self.conf.influxdb.db_per_tenant):
+                return None
+            else:
+                raise
+
     def list_metrics(self, tenant_id, region, name, dimensions, offset,
                      limit, start_timestamp=None, end_timestamp=None):
 
@@ -314,7 +342,7 @@ class MetricsRepository(metrics_repository.AbstractMetricsRepository):
             if offset:
                 query += ' offset {}'.format(int(offset) + 1)
 
-            result = self.influxdb_client.query(query)
+            result = self.query_tenant_db(query, tenant_id)
 
             json_metric_list = self._build_serie_metric_list(result,
                                                              tenant_id,
@@ -336,31 +364,6 @@ class MetricsRepository(metrics_repository.AbstractMetricsRepository):
             LOG.exception(ex)
             raise exceptions.RepositoryException(ex)
 
-    def _build_serie_dimension_values_to_v0_11_0(self, series_names, dimension_name):
-        dim_value_set = set()
-        json_dim_value_list = []
-
-        if not series_names:
-            return json_dim_value_list
-        if 'series' not in series_names.raw:
-            return json_dim_value_list
-        if not dimension_name:
-            return json_dim_value_list
-
-        for series in series_names.raw['series']:
-            if 'columns' not in series:
-                continue
-            if u'values' not in series:
-                continue
-            for value in series[u'values']:
-                dim_value_set.add(value[0])
-
-        for value in dim_value_set:
-            json_dim_value_list.append({u'dimension_value': value})
-
-        json_dim_value_list = sorted(json_dim_value_list)
-        return json_dim_value_list
-
     def _build_serie_dimension_values_from_v0_11_0(self, series_names, dimension_name):
         '''In InfluxDB v0.11.0 the SHOW TAG VALUES output changed.
         See, https://github.com/influxdata/influxdb/blob/master/CHANGELOG.md#v0110-2016-03-22
@@ -392,7 +395,7 @@ class MetricsRepository(metrics_repository.AbstractMetricsRepository):
         for value in dim_value_set:
             json_dim_value_list.append({u'dimension_value': value})
 
-        json_dim_value_list = sorted(json_dim_value_list)
+        json_dim_value_list = sorted(json_dim_value_list, key=lambda x: x[u'dimension_value'])
         return json_dim_value_list
 
     def _build_serie_dimension_names(self, series_names):
@@ -421,46 +424,6 @@ class MetricsRepository(metrics_repository.AbstractMetricsRepository):
         json_dim_name_list = sorted(json_dim_name_list, key=lambda x: x[u'dimension_name'])
         return json_dim_name_list
 
-    def _build_serie_metric_list_to_v0_11_0(self, series_names, tenant_id, region,
-                                            start_timestamp, end_timestamp,
-                                            offset):
-        json_metric_list = []
-
-        if not series_names:
-            return json_metric_list
-
-        if 'series' not in series_names.raw:
-            return json_metric_list
-
-        metric_id = 0
-        if offset:
-            metric_id = int(offset) + 1
-
-        for series in series_names.raw['series']:
-
-            for tag_values in series[u'values']:
-
-                dimensions = {
-                    name: value
-                    for name, value in zip(series[u'columns'], tag_values)
-                    if value and not name.startswith(u'_')
-                }
-
-                if self._has_measurements(tenant_id,
-                                          region,
-                                          series[u'name'],
-                                          dimensions,
-                                          start_timestamp,
-                                          end_timestamp):
-                    metric = {u'id': str(metric_id),
-                              u'name': series[u'name'],
-                              u'dimensions': dimensions}
-                    metric_id += 1
-
-                    json_metric_list.append(metric)
-
-        return json_metric_list
-
     def _build_serie_metric_list_from_v0_11_0(self, series_names, tenant_id, region,
                                               start_timestamp, end_timestamp, offset):
         '''In InfluxDB v0.11.0 the SHOW SERIES output changed.
@@ -535,7 +498,7 @@ class MetricsRepository(metrics_repository.AbstractMetricsRepository):
             entry = {u'name': name[0]}
             json_metric_list.append(entry)
 
-        json_metric_list = sorted(json_metric_list)
+        json_metric_list = sorted(json_metric_list, key=lambda k: k['name'])
         return json_metric_list
 
     def _get_dimensions(self, tenant_id, region, name, dimensions):
@@ -577,7 +540,7 @@ class MetricsRepository(metrics_repository.AbstractMetricsRepository):
                 dimensions = self._get_dimensions(tenant_id, region, name, dimensions)
                 query += " slimit 1"
 
-            result = self.influxdb_client.query(query)
+            result = self.query_tenant_db(query, tenant_id)
 
             if not result:
                 return json_measurement_list
@@ -591,7 +554,9 @@ class MetricsRepository(metrics_repository.AbstractMetricsRepository):
                     measurements_list = []
                     for point in serie['values']:
                         value_meta = rest_utils.from_json(point[2]) if point[2] else {}
-                        timestamp = point[0][:19] + '.' + point[0][20:-1].ljust(3, '0') + 'Z'
+                        datetime = point[0][:19]
+                        fraction = point[0][20:-1].ljust(3, '0')
+                        timestamp = datetime + '.' + fraction[:3] + 'Z'
 
                         measurements_list.append([timestamp,
                                                   point[1],
@@ -653,7 +618,7 @@ class MetricsRepository(metrics_repository.AbstractMetricsRepository):
             query = self._build_show_measurements_query(dimensions, None, tenant_id,
                                                         region)
 
-            result = self.influxdb_client.query(query)
+            result = self.query_tenant_db(query, tenant_id)
 
             json_name_list = self._build_measurement_name_list(result)
             return json_name_list
@@ -679,7 +644,7 @@ class MetricsRepository(metrics_repository.AbstractMetricsRepository):
                 dimensions = self._get_dimensions(tenant_id, region, name, dimensions)
                 query += " slimit 1"
 
-            result = self.influxdb_client.query(query)
+            result = self.query_tenant_db(query, tenant_id)
 
             if not result:
                 return json_statistics_list
@@ -844,13 +809,13 @@ class MetricsRepository(metrics_repository.AbstractMetricsRepository):
                      reason, reason_data, sub_alarms, tenant_id
               from alarm_state_history
               """
-
+            tenant_id = tenant_id if PY3 else tenant_id.encode('utf-8')
             where_clause = (
-                " where tenant_id = '{}' ".format(tenant_id.encode('utf8')))
+                " where tenant_id = '{}' ".format(tenant_id))
 
             alarm_id_where_clause_list = (
-                [" alarm_id = '{}' ".format(id.encode('utf8'))
-                 for id in alarm_id_list])
+                [" alarm_id = '{}' ".format(alarm_id if PY3 else alarm_id.encode('utf8'))
+                 for alarm_id in alarm_id_list])
 
             alarm_id_where_clause = " or ".join(alarm_id_where_clause_list)
 
@@ -871,7 +836,7 @@ class MetricsRepository(metrics_repository.AbstractMetricsRepository):
 
             query += where_clause + time_clause + offset_clause + limit_clause
 
-            result = self.influxdb_client.query(query)
+            result = self.query_tenant_db(query, tenant_id)
 
             if not result:
                 return json_alarm_history_list
@@ -915,12 +880,15 @@ class MetricsRepository(metrics_repository.AbstractMetricsRepository):
         return int((dt - datetime(1970, 1, 1)).total_seconds() * 1000)
 
     def list_dimension_values(self, tenant_id, region, metric_name,
-                              dimension_name):
+                              dimension_name, start_timestamp=None,
+                              end_timestamp=None):
         try:
             query = self._build_show_tag_values_query(metric_name,
                                                       dimension_name,
-                                                      tenant_id, region)
-            result = self.influxdb_client.query(query)
+                                                      tenant_id, region,
+                                                      start_timestamp,
+                                                      end_timestamp)
+            result = self.query_tenant_db(query, tenant_id)
             json_dim_name_list = self._build_serie_dimension_values(
                 result, dimension_name)
             return json_dim_name_list
@@ -928,11 +896,14 @@ class MetricsRepository(metrics_repository.AbstractMetricsRepository):
             LOG.exception(ex)
             raise exceptions.RepositoryException(ex)
 
-    def list_dimension_names(self, tenant_id, region, metric_name):
+    def list_dimension_names(self, tenant_id, region, metric_name,
+                             start_timestamp=None, end_timestamp=None):
         try:
             query = self._build_show_tag_keys_query(metric_name,
-                                                    tenant_id, region)
-            result = self.influxdb_client.query(query)
+                                                    tenant_id, region,
+                                                    start_timestamp,
+                                                    end_timestamp)
+            result = self.query_tenant_db(query, tenant_id)
             json_dim_name_list = self._build_serie_dimension_names(result)
             return json_dim_name_list
         except Exception as ex:
