@@ -15,11 +15,6 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
-from __future__ import unicode_literals
-
 from sqlalchemy import Column
 from sqlalchemy.ext import compiler
 from sqlalchemy.sql import expression
